"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const camelcase_1 = __importDefault(require("camelcase"));
const path = __importStar(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const sanitizeIdentifier_1 = __importDefault(require("./sanitizeIdentifier"));
const FileLevelValueSymbol = typescript_1.default.SymbolFlags.Variable
    | typescript_1.default.SymbolFlags.Function
    | typescript_1.default.SymbolFlags.Class
    | typescript_1.default.SymbolFlags.Enum;
function deriveUniqueName(node, program, pascalCase = false) {
    let name = deriveNameFromSourceFile(node.getSourceFile());
    if (pascalCase) {
        name = camelcase_1.default(name, { pascalCase: true });
    }
    const usedNames = resolveNamesInScope(node, program);
    if (!usedNames.has(name)) {
        return typescript_1.default.createIdentifier(name);
    }
    for (let i = 1; i < 32; i++) {
        if (!usedNames.has(`${name}_${i}`)) {
            return typescript_1.default.createIdentifier(`${name}_${i}`);
        }
    }
    // NOTE: We don't use this method as primary because it doesn't respect
    // the suggested name in the case of function and class declaration.
    return typescript_1.default.createFileLevelUniqueName(name);
}
exports.default = deriveUniqueName;
function deriveNameFromSourceFile(sourceFile) {
    let name = path.basename(sourceFile.fileName).replace(/\.[jt]sx?$/, '');
    if (name === 'index') {
        name = path.basename(path.dirname(sourceFile.fileName));
    }
    return sanitizeIdentifier_1.default(name);
}
function resolveNamesInScope(node, program) {
    return program.getTypeChecker()
        .getSymbolsInScope(node, FileLevelValueSymbol)
        .reduce((acc, sym) => acc.add(sym.name), new Set());
}
//# sourceMappingURL=deriveUniqueName.js.map